@@ -64,10 +64,9 @@ struct CreateRoom
     //! server will overwrite the following keys: creator, room_version. Future versions of the
     //! specification may allow the server to overwrite other keys.
     std::optional<events::state::Create> creation_content;
-};
 
-void
-to_json(nlohmann::json &obj, const CreateRoom &request);
+    friend void to_json(nlohmann::json &obj, const CreateRoom &request);
+};
 
 namespace login_identifier {
 //! The user is identified by their Matrix ID.
@@ -118,10 +117,9 @@ struct Login
     //! A display name to assign to the newly-created device.
     //! Ignored if device_id corresponds to a known device.
     std::string initial_device_display_name;
-};
 
-void
-to_json(nlohmann::json &obj, const Login &request);
+    friend void to_json(nlohmann::json &obj, const Login &request);
+};
 
 /// @brief Request payload for
 /// `POST /_matrix/client/r0/{register,account/password}/email/requestToken`
@@ -144,10 +142,9 @@ struct RequestEmailToken
     //! email (e.g. a reminder) to be sent. If they do not, the server should respond with success
     //! but not resend the email.
     int send_attempt = 0;
-};
 
-void
-to_json(nlohmann::json &obj, const RequestEmailToken &request);
+    friend void to_json(nlohmann::json &obj, const RequestEmailToken &request);
+};
 
 /// @brief Request payload for
 /// `POST /_matrix/client/r0/{register,account/password}/msisdn/requestToken`
@@ -186,10 +183,9 @@ struct IdentitySubmitToken
     std::string client_secret;
     //! Required. The token generated by the requestToken call and emailed to the user.
     std::string token;
-};
 
-void
-to_json(nlohmann::json &obj, const IdentitySubmitToken &request);
+    friend void to_json(nlohmann::json &obj, const IdentitySubmitToken &request);
+};
 
 //! Request payload for the `PUT /_matrix/client/r0/sendToDevice/{eventType}/{transcationID}`
 template<typename EventContent>
@@ -199,20 +195,18 @@ using ToDeviceMessages = std::map<mtx::identifiers::User, std::map<std::string,
 struct AvatarUrl
 {
     std::string avatar_url;
-};
 
-void
-to_json(nlohmann::json &obj, const AvatarUrl &request);
+    friend void to_json(nlohmann::json &obj, const AvatarUrl &request);
+};
 
 //! Request payload for the `PUT /_matrix/client/r0/profile/{userId}/displayname` endpoint.
 struct DisplayName
 {
     //! The new display name for this device. If not given, the display name is unchanged.
     std::string displayname;
-};
 
-void
-to_json(nlohmann::json &obj, const DisplayName &request);
+    friend void to_json(nlohmann::json &obj, const DisplayName &request);
+};
 
 //! Request payload for the `POST /_matrix/client/r0/rooms/{roomId}/invite` endpoint as well as ban,
 //! unban and kick.
@@ -222,10 +216,9 @@ struct RoomMembershipChange
 
     //! optional kick, invite or ban reason
     std::string reason;
-};
 
-void
-to_json(nlohmann::json &obj, const RoomMembershipChange &request);
+    friend void to_json(nlohmann::json &obj, const RoomMembershipChange &request);
+};
 
 //! Request payload for the `PUT /_matrix/client/r0/rooms/{roomId}/typing/{userId}` endpoint.
 struct TypingNotification
@@ -234,19 +227,17 @@ struct TypingNotification
     bool typing = false;
     //! The length of time in milliseconds to mark this user as typing.
     uint64_t timeout = 0;
-};
 
-void
-to_json(nlohmann::json &obj, const TypingNotification &request);
+    friend void to_json(nlohmann::json &obj, const TypingNotification &request);
+};
 
 //! Request payload for the `PUT /_matrix/client/r0/devices/{deviceId}` endpoint.
 struct DeviceUpdate
 {
     std::string display_name;
-};
 
-void
-to_json(nlohmann::json &obj, const DeviceUpdate &request);
+    friend void to_json(nlohmann::json &obj, const DeviceUpdate &request);
+};
 
 //! Request payload for the `PUT /_matrix/client/r0/directory/list/room/{roomId}` endpoint
 struct PublicRoomVisibility
@@ -254,20 +245,18 @@ struct PublicRoomVisibility
     //! The new visibility setting for the room. Defaults to 'public'. One of: ["private",
     //! "public"]
     common::RoomVisibility visibility;
-};
 
-void
-to_json(nlohmann::json &obj, const PublicRoomVisibility &request);
+    friend void to_json(nlohmann::json &obj, const PublicRoomVisibility &request);
+};
 
 struct PublicRoomsFilter
 {
     //! A string to search for in the room metadata,
     //! e.g. name, topic, canonical alias etc. (Optional).
     std::string generic_search_term;
-};
 
-void
-to_json(nlohmann::json &obj, const PublicRoomsFilter &req);
+    friend void to_json(nlohmann::json &obj, const PublicRoomsFilter &req);
+};
 
 //! Request payload for the `POST /_matrix/client/r0/publicRooms` endpoint.
 struct PublicRooms
@@ -287,20 +276,22 @@ struct PublicRooms
     //! The specific third party network/protocol to request from
     //! the homeserver. Can only be used if include_all_networks is false.
     std::string third_party_instance_id;
+
+    friend void to_json(nlohmann::json &obj, const PublicRooms &request);
 };
 
-void
-to_json(nlohmann::json &obj, const PublicRooms &request);
-
+//! An empty response.
+///
+/// Some endpoints return this on success.
 struct Empty
-{};
-
-inline void
-to_json(nlohmann::json &, const Empty &)
-{}
+{
+    friend inline void to_json(nlohmann::json &, const Empty &) {}
+};
 
+//! A successfull logout.
 using Logout = Empty;
 
+//! A signed onetime key. Usually curve25519.
 struct SignedOneTimeKey
 {
     //! Required. The unpadded Base64-encoded 32-byte Curve25519 public key.
@@ -311,10 +302,11 @@ struct SignedOneTimeKey
     //! Required. Signatures of the key object.
     //! The signature is calculated using the process described at Signing JSON.
     std::map<std::string, std::map<std::string, std::string>> signatures;
-};
-void
-to_json(nlohmann::json &obj, const SignedOneTimeKey &);
 
+    friend void to_json(nlohmann::json &obj, const SignedOneTimeKey &);
+};
+
+//! Upload a mix of device, one time and fallback keys.
 struct UploadKeys
 {
     //! Identity keys for the device.
@@ -338,13 +330,14 @@ struct UploadKeys
     //!
     //! May be absent if a new fallback key is not required.
     std::map<std::string, std::variant<std::string, SignedOneTimeKey>> fallback_keys;
+
+    friend void to_json(nlohmann::json &obj, const UploadKeys &);
 };
 
-void
-to_json(nlohmann::json &obj, const UploadKeys &);
-
-constexpr uint64_t DEFAULT_DOWNLOAD_TIMEOUT = 10 * 1000; // 10 seconds
+//! The default timeout for a download.
+constexpr uint64_t DEFAULT_DOWNLOAD_TIMEOUT = 10UL * 1000; // 10 seconds
 
+//! Query the keys of devices of specific users.
 struct QueryKeys
 {
     //! The time (in milliseconds) to wait when downloading keys from remote servers.
@@ -359,22 +352,22 @@ struct QueryKeys
     //! This allows the server to ensure its response contains the keys
     //! advertised by the notification in that sync.
     std::string token;
+
+    friend void to_json(nlohmann::json &obj, const QueryKeys &);
 };
 
-void
-to_json(nlohmann::json &obj, const QueryKeys &);
-
+//! Claim onetime keys of devices of specific users.
 struct ClaimKeys
 {
     //! The time (in milliseconds) to wait when downloading keys from remote servers.
     uint64_t timeout = DEFAULT_DOWNLOAD_TIMEOUT;
     //! The keys to be claimed. A map from user ID, to a map from device ID to algorithm name.
     std::map<std::string, std::map<std::string, std::string>> one_time_keys;
+
+    friend void to_json(nlohmann::json &obj, const ClaimKeys &request);
 };
 
-void
-to_json(nlohmann::json &obj, const ClaimKeys &request);
-
+//! Upload new signatures for a device or cross-signing key.
 struct KeySignaturesUpload
 {
     //! map from user_id to either a map of device id to DeviceKey with new signatures or the
@@ -383,10 +376,9 @@ struct KeySignaturesUpload
       std::string,
       std::map<std::string, std::variant<mtx::crypto::DeviceKeys, mtx::crypto::CrossSigningKeys>>>
       signatures;
-};
 
-void
-to_json(nlohmann::json &obj, const KeySignaturesUpload &req);
+    friend void to_json(nlohmann::json &obj, const KeySignaturesUpload &req);
+};
 
 //! Upload cross signing keys
 struct DeviceSigningUpload
@@ -399,11 +391,11 @@ struct DeviceSigningUpload
     //! Optional. The user's user-signing key. Must be signed by the accompanying master key, or by
     //! the user's most recently uploaded master key if no master key is included in the request.
     std::optional<mtx::crypto::CrossSigningKeys> user_signing_key;
+
+    friend void to_json(nlohmann::json &obj, const DeviceSigningUpload &req);
 };
 
-void
-to_json(nlohmann::json &obj, const DeviceSigningUpload &req);
-
+//! Information about a pusher.
 struct PusherData
 {
     //! Required if `kind` is http. The URL to use to send notifications to.
@@ -414,10 +406,9 @@ struct PusherData
     //! defined in the Push Gateway Specification. Currently the only format available is
     //! 'event_id_only'.
     std::string format;
-};
 
-void
-to_json(nlohmann::json &obj, const PusherData &data);
+    friend void to_json(nlohmann::json &obj, const PusherData &data);
+};
 
 //! Request payload for the `POST /_matrix/client/r0/pushers/set` endpoint.
 struct SetPusher
@@ -445,9 +436,8 @@ struct SetPusher
     PusherData data;
     //! If true, add another pusher instead of updating an existing one.
     bool append = false;
-};
 
-void
-to_json(nlohmann::json &obj, const SetPusher &req);
+    friend void to_json(nlohmann::json &obj, const SetPusher &req);
+};
 } // namespace requests
 } // namespace mtx
