@@ -916,8 +916,9 @@ mtx::crypto::encrypt_exported_sessions(const mtx::crypto::ExportedSessionKeys &k
 {
     const auto plaintext = json(keys).dump();
 
-    auto nonce = create_buffer(AES_BLOCK_SIZE);
-    nonce[15 - 63 % 8] &= ~(1UL << (63 / 8));
+    auto nonce                  = create_buffer(AES_BLOCK_SIZE);
+    constexpr std::uint8_t mask = static_cast<std::uint8_t>(~(1U << (63 / 8)));
+    nonce[15 - 63 % 8] &= mask;
 
     auto salt = create_buffer(pwhash_SALTBYTES);
 
